module IOUtils (Color, colorToString, tupToString, v3ToColor) where

import Linear.V3
import Maths

type Color = (Double, Double, Double)

-- convert color to string, make sure each value is between 0 and 256
colorToString :: Int -> Color -> IO ()
colorToString samples (x, y, z) = tupToString (xr, yr, zr)
  where
    samplesD = fromIntegral samples
    scale = 1.0 / samplesD
    xr = truncate $ (clamp (sqrt (x * scale)) 0.0 0.999) * 256.0
    yr = truncate $ (clamp (sqrt (y * scale)) 0.0 0.999) * 256.0
    zr = truncate $ (clamp (sqrt (z * scale)) 0.0 0.999) * 256.0

tupToString :: (Show a) => (a, a, a) -> IO ()
tupToString (x, y, z) = putStrLn (show x ++ " " ++ show y ++ " " ++ show z)

v3ToColor :: V3 Double -> Color
v3ToColor (V3 x y z) = (x, y, z)
