module Ray
  ( Ray (..),
    rayAt,
    calculateRayNormal,
    buildRay,
    buildRayAA,
  )
where

import Camera
import Control.Monad.Random
import Linear.V3
import Linear.Vector
import Maths

data Ray = Ray {origin :: V3 Double, direction :: V3 Double}

-- P(t) = A + tB; A is origin, B is direction
rayAt :: Ray -> Double -> V3 Double
rayAt (Ray origin dir) t = origin + (t *^ dir)

-- get the normal of the point at t
calculateRayNormal :: Ray -> Double -> V3 Double
calculateRayNormal r t = unitVector (rayAt r t ^-^ V3 0.0 0.0 1.0)

buildRayAA :: (RandomGen g) => Double -> Double -> Double -> Double -> Camera -> Rand g Ray
buildRayAA u v w h camera = do
  ur <- randRange 0 1
  vr <- randRange 0 1
  return $ buildRay ((u + ur) / w) ((v + vr) / h) camera

buildRay :: Double -> Double -> Camera -> Ray
buildRay u v (Camera o ll h vv) = Ray o (ll ^+^ (u *^ h) ^+^ (v *^ vv) ^-^ o)
